@@ -1,4 +1,4 @@
-# BSD 3-Clause License; see https://github.com/scikit-hep/uproot4/blob/main/LICENSE
+# BSD 3-Clause License; see https://github.com/scikit-hep/uproot5/blob/main/LICENSE
 
 """
 This module defines utilities for modeling C++ objects as Python objects and the
@@ -16,10 +16,11 @@ The :doc:`uproot.model.UnknownClass` and
 not be modeled, either because the class has no streamer or no streamer for its
 version.
 """
-
+from __future__ import annotations
 
 import re
 import sys
+import threading
 import weakref
 
 import numpy
@@ -103,9 +104,32 @@ def reset_classes():
     reload(uproot.models.TH)
     reload(uproot.models.TGraph)
     reload(uproot.models.TMatrixT)
+    reload(uproot.models.TTime)
 
 
-_classname_regularize = re.compile(r"\s*(<|>|::)\s*")
+_root_alias_to_c_primitive = {
+    "Bool_t": "bool",
+    "Char_t": "char",
+    "UChar_t": "unsigned char",
+    "Short_t": "short",
+    "UShort_t": "unsigned short",
+    "Int_t": "int",
+    "UInt_t": "unsigned int",
+    "Long_t": "long",
+    "ULong_t": "unsigned long",
+    "Long64_t": "long long",
+    "ULong64_t": "unsigned long long",
+    "Size_t": "size_t",
+    "Float_t": "float",
+    "Double_t": "double",
+    "LongDouble_t": "long double",
+}
+
+_classname_regularize = re.compile(r"\s*(<|>|,|::)\s*")
+_classname_regularize_type = re.compile(
+    r"[<,](" + "|".join([re.escape(p) for p in _root_alias_to_c_primitive]) + r")[>,]"
+)
+
 _classname_encode_pattern = re.compile(rb"[^a-zA-Z0-9]+")
 _classname_decode_antiversion = re.compile(rb".*_([0-9a-f][0-9a-f])+_v([0-9]+)$")
 _classname_decode_version = re.compile(rb".*_v([0-9]+)$")
@@ -130,10 +154,20 @@ def classname_regularize(classname):
     If ``classname`` is None, this function returns None. Otherwise, it must be
     a string and it returns a string.
     """
-    if classname is None:
-        return classname
-    else:
-        return re.sub(_classname_regularize, r"\1", classname)
+    if classname is not None:
+        classname = re.sub(_classname_regularize, r"\1", classname)
+
+        m = _classname_regularize_type.search(classname)
+
+        while m is not None:
+            start, stop = m.span(1)
+            token = classname[start:stop]
+            replacement = _root_alias_to_c_primitive[token]
+            classname = classname[:start] + replacement + classname[stop:]
+
+            m = _classname_regularize_type.search(classname)
+
+    return classname
 
 
 def classname_decode(encoded_classname):
@@ -186,17 +220,11 @@ def classname_encode(classname, version=None, unknown=False):
     by underscores. Additionally, Python models of C++ classes are prepended
     with ``Model_`` (or ``Unknown_`` if a streamer isn't found).
     """
-    if unknown:
-        prefix = "Unknown_"
-    else:
-        prefix = "Model_"
+    prefix = "Unknown_" if unknown else "Model_"
     if classname.startswith(prefix):
         raise ValueError(f"classname is already encoded: {classname}")
 
-    if version is None:
-        v = ""
-    else:
-        v = "_v" + str(version)
+    v = "" if version is None else "_v" + str(version)
 
     raw = classname.encode()
     out = _classname_encode_pattern.sub(_classname_encode_convert, raw)
@@ -262,9 +290,7 @@ def class_named(classname, version=None, custom_classes=None):
             return versioned_cls
         else:
             raise ValueError(
-                "no class named {} with version {} in {}".format(
-                    classname, version, where
-                )
+                f"no class named {classname} with version {version} in {where}"
             )
 
     else:
@@ -567,8 +593,8 @@ class Model:
 
         The ``cls`` arguments may be Python classes or C++ classname strings to match.
         """
-        cpp_names = [classname_regularize(x) for x in cls if uproot._util.isstr(x)]
-        py_types = tuple(x for x in cls if not uproot._util.isstr(x))
+        cpp_names = [classname_regularize(x) for x in cls if isinstance(x, str)]
+        py_types = tuple(x for x in cls if not isinstance(x, str))
 
         out = []
         for x in getattr(self, "_bases", []):
@@ -586,8 +612,8 @@ class Model:
 
         The ``cls`` arguments may be Python classes or C++ classname strings to match.
         """
-        cpp_names = [classname_regularize(x) for x in cls if uproot._util.isstr(x)]
-        py_types = tuple(x for x in cls if not uproot._util.isstr(x))
+        cpp_names = [classname_regularize(x) for x in cls if isinstance(x, str)]
+        py_types = tuple(x for x in cls if not isinstance(x, str))
 
         if isinstance(self, py_types) or any(self.classname == n for n in cpp_names):
             return True
@@ -647,10 +673,9 @@ class Model:
             index_format (str): Format to use for indexes of the
                 ``awkward.forms.Form``; may be ``"i32"``, ``"u32"``, or
                 ``"i64"``.
-            header (bool): If True, include headers in the Form's ``"uproot"``
-                parameters.
-            tobject_header (bool): If True, include headers for ``TObject``
-                classes in the Form's ``"uproot"`` parameters.
+            header (bool): If True, include header fields of each C++ class.
+            tobject_header (bool): If True, include header fields of each ``TObject``
+                base class.
             breadcrumbs (tuple of class objects): Used to check for recursion.
                 Types that contain themselves cannot be Awkward Arrays because the
                 depth of instances is unknown.
@@ -760,15 +785,22 @@ class Model:
         self._num_bytes = None
         self._instance_version = None
         self._is_memberwise = False
-
         old_breadcrumbs = context.get("breadcrumbs", ())
-        context["breadcrumbs"] = old_breadcrumbs + (self,)
+        context["breadcrumbs"] = (*old_breadcrumbs, self)
 
         self.hook_before_read(chunk=chunk, cursor=cursor, context=context, file=file)
-
+        forth_obj = uproot._awkwardforth.get_forth_obj(context)
+        if forth_obj is not None:
+            forth_stash = uproot._awkwardforth.Node(
+                name=f"wrong-instance-version {uproot._awkwardforth.get_first_key_number(context)}",
+                form_details={"offsets": "i64"},
+            )
         if context.get("reading", True):
+            temp_index = cursor._index
             self.read_numbytes_version(chunk, cursor, context)
-
+            length = cursor._index - temp_index
+            if length != 0 and forth_obj is not None:
+                forth_stash.pre_code.append(f"{length} stream skip\n")
             if (
                 issubclass(cls, VersionedModel)
                 and self._instance_version != classname_version(cls.__name__)
@@ -780,7 +812,10 @@ class Model:
                 ):
                     cursor.move_to(self._cursor.index)
                     context["breadcrumbs"] = old_breadcrumbs
-                    return correct_cls.read(
+                    if forth_obj is not None:
+                        forth_obj.add_node(forth_stash)
+                        forth_obj.push_active_node(forth_stash)
+                    temp_var = correct_cls.read(
                         chunk,
                         cursor,
                         context,
@@ -789,21 +824,36 @@ class Model:
                         parent,
                         concrete=concrete,
                     )
+                    if forth_obj is not None:
+                        forth_obj.pop_active_node()
+
+                    return temp_var
 
         if context.get("in_TBranch", False):
+            # AwkwardForth testing A: test_0637's 01,02,05,08,09,11,12,13,15,16,29,35,38,39,44,45,46,47,49,50,52,56
             if self._num_bytes is None and self._instance_version != self.class_version:
                 self._instance_version = None
                 cursor = self._cursor
+                if forth_obj is not None:
+                    forth_stash._pre_code.pop(-1)
 
             elif self._instance_version == 0:
+                if forth_obj is not None:
+                    forth_stash.pre_code.append("4 stream skip\n")
                 cursor.skip(4)
 
         if context.get("reading", True):
             self.hook_before_read_members(
                 chunk=chunk, cursor=cursor, context=context, file=file
             )
-
+            if forth_obj is not None:
+                key = uproot._awkwardforth.get_first_key_number(context)
+                forth_stash.name = f"start-of-model {key}"
+                forth_obj.add_node(forth_stash)
+                forth_obj.push_active_node(forth_stash)
             self.read_members(chunk, cursor, context, file)
+            if forth_obj is not None:
+                forth_obj.pop_active_node()
 
             self.hook_after_read_members(
                 chunk=chunk, cursor=cursor, context=context, file=file
@@ -861,7 +911,6 @@ class Model:
         :doc:`uproot.model.Model` has an empty ``read_members`` method; this
         *must* be overridden by subclasses.
         """
-        pass
 
     def check_numbytes(self, chunk, cursor, context):
         """
@@ -927,7 +976,6 @@ class Model:
         Called in :ref:`uproot.model.Model.read`, before any data have been
         read.
         """
-        pass
 
     def hook_before_read_members(self, **kwargs):
         """
@@ -935,7 +983,6 @@ class Model:
         :ref:`uproot.model.Model.read_numbytes_version` and before
         :ref:`uproot.model.Model.read_members`.
         """
-        pass
 
     def hook_after_read_members(self, **kwargs):
         """
@@ -943,7 +990,6 @@ class Model:
         :ref:`uproot.model.Model.read_members` and before
         :ref:`uproot.model.Model.check_numbytes`.
         """
-        pass
 
     def hook_before_postprocess(self, **kwargs):
         """
@@ -951,7 +997,6 @@ class Model:
         :ref:`uproot.model.Model.check_numbytes` and before
         :ref:`uproot.model.Model.postprocess`.
         """
-        pass
 
     def _to_writable_postprocess(self, original):
         pass
@@ -968,9 +1013,8 @@ class Model:
                         cls = versioned_cls
                         break
 
-            elif unversioned is not None:
-                if unversioned.writable:
-                    cls = unversioned
+            elif unversioned is not None and unversioned.writable:
+                cls = unversioned
 
         if cls is None:
             raise NotImplementedError(
@@ -1019,9 +1063,7 @@ class Model:
 
     def _serialize(self, out, header, name, tobject_flags):
         raise NotImplementedError(
-            "can't write {} instances yet ('serialize' method not implemented)".format(
-                type(self).__name__
-            )
+            f"can't write {type(self).__name__} instances yet ('serialize' method not implemented)"
         )
 
     def serialize(self, name=None):
@@ -1080,14 +1122,29 @@ class VersionedModel(Model):
                 "writable": self.writable,
                 "behaviors": self.behaviors,
             },
-            dict(self.__dict__),
+            {
+                k: (
+                    _LockPlaceholder()
+                    if isinstance(v, _LockPlaceholder.lock_type)
+                    else v
+                )
+                for k, v in self.__dict__.items()
+            },
         )
 
     def __setstate__(self, state):
         class_data, instance_data = state
+        instance_data = {
+            k: threading.Lock() if isinstance(v, _LockPlaceholder) else v
+            for k, v in instance_data.items()
+        }
         self.__dict__.update(instance_data)
 
 
+class _LockPlaceholder:
+    lock_type = type(threading.Lock())
+
+
 class DispatchByVersion:
     """
     A Python class that models all versions of a ROOT C++ class by maintaining
@@ -1263,7 +1320,7 @@ class DispatchByVersion:
         import uproot.deserialization
 
         # Ignores context["reading"], because otherwise, there would be nothing to do.
-
+        start_index = cursor._index
         (
             num_bytes,
             version,
@@ -1272,6 +1329,18 @@ class DispatchByVersion:
 
         versioned_cls = cls.class_of_version(version)
 
+        forth_obj = uproot._awkwardforth.get_forth_obj(context)
+        if forth_obj is not None:
+            forth_stash = uproot._awkwardforth.Node(
+                f"dispatch-by-version {uproot._awkwardforth.get_first_key_number(context)}",
+                form_details={"offsets": "i64"},
+            )
+            bytes_skipped = cursor._index - start_index
+            forth_stash.pre_code.append(f"{bytes_skipped} stream skip \n")
+
+            forth_obj.add_node(forth_stash)
+            forth_obj.push_active_node(forth_stash)
+
         if versioned_cls is not None:
             pass
 
@@ -1283,16 +1352,13 @@ class DispatchByVersion:
 
         else:
             raise ValueError(
-                """Unknown version {} for class {} that cannot be skipped """
+                f"""Unknown version {version} for class {classname_decode(cls.__name__)[0]} that cannot be skipped """
                 """because its number of bytes is unknown.
-""".format(
-                    version,
-                    classname_decode(cls.__name__)[0],
-                )
+"""
             )
 
         # versioned_cls.read starts with numbytes_version again because move=False (above)
-        return cls.postprocess(
+        temp_var = cls.postprocess(
             versioned_cls.read(
                 chunk, cursor, context, file, selffile, parent, concrete=concrete
             ),
@@ -1302,6 +1368,11 @@ class DispatchByVersion:
             file,
         )
 
+        if forth_obj is not None:
+            forth_obj.pop_active_node()
+
+        return temp_var
+
     @classmethod
     def postprocess(cls, self, chunk, cursor, context, file):
         """
@@ -1460,11 +1531,9 @@ class UnknownClass(Model):
 
         else:
             raise ValueError(
-                """unknown class {} that cannot be skipped because its """
-                """number of bytes is unknown
-in file {}""".format(
-                    self.classname, file.file_path
-                )
+                f"""unknown class {self.classname} that cannot be skipped because its """
+                f"""number of bytes is unknown
+in file {file.file_path}"""
             )
 
 
@@ -1593,17 +1662,13 @@ class UnknownClassVersion(VersionedModel):
 
         else:
             raise ValueError(
-                """class {} with unknown version {} cannot be skipped """
-                """because its number of bytes is unknown
-in file {}""".format(
-                    self.classname, self._instance_version, file.file_path
-                )
+                f"""class {self.classname} with unknown version {self._instance_version} cannot be skipped """
+                f"""because its number of bytes is unknown
+in file {file.file_path}"""
             )
 
     def __repr__(self):
-        return "<{} with unknown version {} at 0x{:012x}>".format(
-            self.classname, self._instance_version, id(self)
-        )
+        return f"<{self.classname} with unknown version {self._instance_version} at 0x{id(self):012x}>"
 
 
 class DynamicModel(VersionedModel):
